﻿using EPiServer.Common;
using EPiServer.Common.Cache;
using EPiServer.Common.Data;
using EPiServer.Common.Security;

namespace ExampleProjects.Movie
{
    public static class MovieHandler
    {
        #region Cache delegates
        internal delegate Movie GetMovieDelegate(int id);
        internal delegate PagedCollectionResultSet GetMoviesByAuthorDelegate(
            IUser author, int page, int pageSize);

        internal static GetMovieDelegate 
            _getMovieDelegate = Data.MovieFactory.GetMovie;
        internal static GetMoviesByAuthorDelegate 
            _getMoviesByAuthorDelegate = Data.MovieFactory.GetMoviesByAuthor;
        #endregion

        #region Events
public static event EPiServerCommonEventHandler MovieAdded;
public static event EPiServerCommonEventHandler MovieUpdated;
public static event EPiServerCommonEventHandler MovieRemoved;
        #endregion

        #region Event firing
        internal static void OnMovieAdded(Movie movie)
        {
            if (MovieAdded == null)
                return;

            EPiServerCommonEventArgs eventArgs = 
                CreateEventArgs(movie, "movie_added");
            MovieAdded(MovieModule.Instance.UniqueName, eventArgs);
        }

        private static EPiServerCommonEventArgs 
            CreateEventArgs(Movie movie, string eventName)
        {
            return new EPiServerCommonEventArgs(
                movie.ID.ToString(), eventName,
                SiteHandler.CurrentSite, movie);
        }

        internal static void OnMovieUpdated(Movie movie)
        {
            if(MovieUpdated == null)
                return;

            EPiServerCommonEventArgs eventArgs = 
                CreateEventArgs(movie, "movie_updated");
            MovieUpdated(MovieModule.Instance.UniqueName, eventArgs);
        }

        internal static void OnMovieRemoved(Movie movie)
        {
            if (MovieRemoved == null)
                return;

            EPiServerCommonEventArgs eventArgs = 
                CreateEventArgs(movie, "movie_removed");
            MovieRemoved(MovieModule.Instance.UniqueName, eventArgs);
        }
        #endregion


        public static Movie GetMovie(int id)
        {
            if (!EntityValidator.IsValidId(id))
                return null;

            object[] getterArgs = new object[] { id };
            string[] cacheKey = Movie.ConstructCacheKey(id);
            return (Movie)CacheHandler.GetCachedObject(
                _getMovieDelegate, getterArgs, cacheKey);
        }

        public static Movie AddMovie(Movie movie)
        {
            EntityValidator.ValidateIsNotNull(movie, "movie");

            Movie addedMovie = Data.MovieFactory.AddMovie(movie);

            ClearGetMoviesByAuthorCache(movie.Author.ID);

            return addedMovie;
        }

        private static void ClearGetMoviesByAuthorCache(int authorID)
        {
            string[] cacheKey =
                GetMoviesByAuthorBaseCacheKey(authorID);
            CacheHandler.RemoveCachedObject(cacheKey);
        }

        public static void UpdateMovie(Movie movie)
        {
            EntityValidator.ValidateIsCommittedEntity(movie, "movie");
            
            Data.MovieFactory.UpdateMovie(movie);

            ClearGetMoviesByAuthorCache(movie.Author.ID);
        }

        public static void RemoveMovie(Movie movie)
        {
            EntityValidator.ValidateIsCommittedEntity(movie, "movie");
            
            Data.MovieFactory.RemoveMovie(movie);

            ClearGetMoviesByAuthorCache(movie.Author.ID);
        }

        public static MovieCollection GetMoviesByAuthor(
            IUser author, int page, int pageSize, out int totalItems)
        {
            EntityValidator.ValidateIsCommittedEntity(author, "author");

            object[] getterArgs = new object[3];
            getterArgs[0] = author;
            getterArgs[1] = page;
            getterArgs[2] = pageSize;

            string[] baseCacheKey = new string[3];
            baseCacheKey = GetMoviesByAuthorBaseCacheKey(author.ID);

            string[] cacheKey = new string[5];
            cacheKey[0] = baseCacheKey[0];
            cacheKey[1] = baseCacheKey[1];
            cacheKey[2] = baseCacheKey[2];
            cacheKey[3] = pageSize.ToString();
            cacheKey[4] = page.ToString();

            PagedCollectionResultSet pagedMovies = 
                (PagedCollectionResultSet)CacheHandler.GetCachedObject(
                _getMoviesByAuthorDelegate, getterArgs, cacheKey);

            totalItems = pagedMovies.TotalItems;

            return (MovieCollection) pagedMovies.Collection;
        }

        private static string[] 
            GetMoviesByAuthorBaseCacheKey(int authorID)
        {
            string[] cacheKey = new string[3];
            cacheKey[0] = MovieModule.Instance.UniqueName;
            cacheKey[1] = "GetMoviesByAuthor";
            cacheKey[2] = authorID.ToString();

            return cacheKey;
        }
    }
}
